﻿' K8055 Class for Visual Basic 2008.NET
'    Copyright (c) 2011   Vanderhaegen Bart
'
'
' I have created this class to provide an easy access to the Velleman K8055 card.
' This class is provided as it is, you should use it at your own risk.
'
' Currently the class only suppports the use of a single K8055 card. The original
' DLL from Velleman supports up to 4 cards. You can bypass this problem by creating
' 4 instances of the class: 4 different objects. One for each K8055 card.
'
'
' I advise you to use this class as an instant variable of your form. Doing so you
' can access the object of this class from different subs/functions in your form
' class. This is the minimal code you should use to get the card working:
'
'       Public Class frmMain
'         Private card1 As K8055
'
'         Private Sub frmMain_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
'               card1 = new K8055()
'               card1.connect(0)
'         End Sub
'       End Class
'
'
' The methods in this class are named slightly different than the ones in the 
' K8055.dll. This shouldn't be a big problem. The functions you can use are
' Explained on by website http://www.bvsystems.be/k8055class.php
'
'
'
Public Class K8055

    ' Constants with information about the K8055 card
    ' Attention: do NOT change these values as they will make your program
    ' crash. I have just added them to make my code as flexible as possible
    Private Const MAX_ANALOG_OUTPUTS = 2
    Private Const MAX_ANALOG_INPUTS = 2
    Private Const MAX_DIGITAL_OUTPUTS = 8
    Private Const MAX_DIGITAL_INPUTS = 5




#Region "DLL Declarations"
    ' Initialisation & Connection
    Private Declare Function OpenDevice Lib "k8055d.dll" (ByVal CardAddress As Integer) As Integer
    Private Declare Sub CloseDevice Lib "k8055d.dll" ()
    Private Declare Function Version Lib "k8055d.dll" () As Integer
    Private Declare Function SearchDevices Lib "k8055d.dll" () As Integer
    Private Declare Function SetCurrentDevice Lib "k8055d.dll" (ByVal CardAddress As Integer) As Integer

    ' Analog Channels
    Private Declare Function ReadAnalogChannel Lib "k8055d.dll" (ByVal Channel As Integer) As Integer
    Private Declare Sub ReadAllAnalog Lib "k8055d.dll" (ByRef Data1 As Integer, ByRef Data2 As Integer)
    Private Declare Sub OutputAnalogChannel Lib "k8055d.dll" (ByVal Channel As Integer, ByVal Data As Integer)
    Private Declare Sub OutputAllAnalog Lib "k8055d.dll" (ByVal Data1 As Integer, ByVal Data2 As Integer)
    Private Declare Sub ClearAnalogChannel Lib "k8055d.dll" (ByVal Channel As Integer)
    Private Declare Sub SetAllAnalog Lib "k8055d.dll" ()
    Private Declare Sub ClearAllAnalog Lib "k8055d.dll" ()
    Private Declare Sub SetAnalogChannel Lib "k8055d.dll" (ByVal Channel As Integer)

    ' Digital Output Channels
    Private Declare Sub WriteAllDigital Lib "k8055d.dll" (ByVal Data As Integer)
    Private Declare Sub ClearDigitalChannel Lib "k8055d.dll" (ByVal Channel As Integer)
    Private Declare Sub ClearAllDigital Lib "k8055d.dll" ()
    Private Declare Sub SetDigitalChannel Lib "k8055d.dll" (ByVal Channel As Integer)
    Private Declare Sub SetAllDigital Lib "k8055d.dll" ()

    ' Digital Input channels
    Private Declare Function ReadDigitalChannel Lib "k8055d.dll" (ByVal Channel As Integer) As Boolean
    Private Declare Function ReadAllDigital Lib "k8055d.dll" () As Integer

    ' Counters
    Private Declare Function ReadCounter Lib "k8055d.dll" (ByVal CounterNr As Integer) As Integer
    Private Declare Sub ResetCounter Lib "k8055d.dll" (ByVal CounterNr As Integer)
    Private Declare Sub SetCounterDebounceTime Lib "k8055d.dll" (ByVal CounterNr As Integer, ByVal DebounceTime As Integer)

#End Region




#Region "Events & Declarations"

    ' Defined events
    Public Event AnalogInputChanged(ByVal channel As Integer, ByVal Value As Integer)
    Public Event digitalInputChanged(ByVal channel As Integer, ByVal Value As Boolean)
    Public Event readCycleCompleted()

    ' State variables. They contain the cached state of the Input Channels
    Private digitalOutputChannels(MAX_DIGITAL_OUTPUTS) As Boolean
    Private digitalInputChannels(MAX_DIGITAL_INPUTS) As Boolean
    Private analogOutputChannels(MAX_ANALOG_OUTPUTS) As Integer
    Private analogInputChannels(MAX_ANALOG_INPUTS) As Integer

    ' Define a Timer that will be checked in a regular Interval
    Private WithEvents readTimer As New Timer()
    Private _timerEnabled As Boolean = False
    Private _timerInterval As Integer = 1000
    Private _cacheEnabled As Boolean = True
    Private _connected As Boolean = False
    Private _initState As Boolean = False

    ' Static function to check if only one instance of the class exists
    Private Shared instanceCount As Integer

#End Region




#Region "Properties"
    ' Sets or returns the state of the timer. When disabled rasing Events are impossible.
    Public Property timerEnabled() As Boolean
        Get
            Return _timerEnabled
        End Get
        Set(ByVal value As Boolean)
            _timerEnabled = value
            If Not _timerEnabled Then _cacheEnabled = False
        End Set
    End Property

    ' Sets or returns the interval of the timer. Ideal default values are 100 - 500 milliseconds.
    Public Property timerInterval() As Integer
        Get
            Return _timerInterval
        End Get
        Set(ByVal value As Integer)
            _timerInterval = value
            readTimer.Interval = _timerInterval
        End Set
    End Property

    ' Returns a boolean to indicate if a K8000 card is connected.
    Public ReadOnly Property isConnected() As Boolean
        Get
            Return _connected
        End Get
    End Property

    ' Enables or Disables input caching. Recommended value: enabled
    Public Property cacheEnabled() As Integer
        Get
            Return _cacheEnabled
        End Get
        Set(ByVal value As Integer)
            _cacheEnabled = value
        End Set
    End Property
#End Region




#Region "Digital Outputs"
    ' Set the value of a Digital channel
    Public Sub setDigitalOutput(ByVal channel As Integer)
        SetDigitalChannel(channel)
        digitalOutputChannels(channel) = True
    End Sub

    ' Clear the value of a Digital channel 
    Public Sub clearDigitalOutput(ByVal channel As Integer)
        ClearDigitalChannel(channel)
        digitalOutputChannels(channel) = False
    End Sub

    ' Get the value of a Digital channel 
    Public Function getDigitalOutput(ByVal channel As Integer)
        Return digitalOutputChannels(channel)
    End Function

    ' Set all the Digital channels
    Public Sub setAllDigitalOutputs()
        SetAllDigital()
        For i As Integer = 1 To MAX_DIGITAL_OUTPUTS
            digitalOutputChannels(i) = True
        Next
    End Sub

    ' Clear all the Digital channels
    Public Sub clearAllDigtalOutputs()
        ClearAllDigital()
        For i As Integer = 1 To MAX_DIGITAL_OUTPUTS
            digitalOutputChannels(i) = False
        Next
    End Sub
#End Region




#Region "Digital Input Channels"
    ' Get the value of a Digital Input channel 
    Public Function getDigitalInput(ByVal channel As Integer) As Boolean
        If _timerEnabled And _cacheEnabled And Not _initState Then
            ' Cache mode enabled
            Return digitalInputChannels(channel)
        Else
            ' Cache mode disabled
            Return ReadDigitalChannel(channel)
        End If
    End Function

    ' Get the value of all Digital Input channels
    Public Function getAllDigitalInput() As Integer
        If _timerEnabled And _cacheEnabled And Not _initState Then
            ' Cache mode enabled
            Dim sum As Integer = 0
            For i As Integer = 1 To MAX_DIGITAL_INPUTS
                If digitalInputChannels(i) Then
                    sum += Math.Pow(2, i - 1)
                End If
            Next
            Return sum
        Else
            ' Cache mode disabled
            Return ReadAllDigital
        End If
    End Function
#End Region




#Region "Counters"
    ' Read Counter
    Public Function readCounter_(ByVal CounterNr As Integer) As Integer
        Return ReadCounter(CounterNr)
    End Function

    ' Reset Counter
    Public Sub resetCounter_(ByVal CounterNr As Integer)
        ResetCounter(CounterNr)
    End Sub

    ' Set Counter DebounceTime
    Public Sub setCounterDebounceTime_(ByVal CounterNr As Integer, ByVal debounceTime As Integer)
        SetCounterDebounceTime(CounterNr, debounceTime)
    End Sub
#End Region




#Region "Connecting K8055"
    ' Initialize the card after creating a new connection
    Private Sub initializeConnection()
        ' Clear any analog outputs.
        clearAllAnalogOutputs()
        clearAllDigtalOutputs()
        _initState = True
    End Sub


    ' Connect to a K8055 card by giving the address directly
    Public Function connect(ByVal cardAddress As Integer) As Boolean
        Return connectDevice(cardAddress)
    End Function

    ' Connect to a K8055 card by giving the state of the jumpers
    Public Function connect(ByVal SK5 As Boolean, ByVal SK6 As Boolean) As Boolean
        Dim cardAddress As Integer
        cardAddress = 3
        If SK5 Then cardAddress -= 1
        If SK6 Then cardAddress -= 2
        Return connectDevice(cardAddress)
    End Function

    ' Private function that will actully do the connection 
    Private Function connectDevice(ByVal CardAddress As Integer) As Boolean
        Dim h As Integer

        If instanceCount = 0 Then
            h = OpenDevice(CardAddress)
            initializeConnection()
            Select Case h
                Case 0, 1, 2, 3
                    _connected = True
                    instanceCount += 1
                    Return True
                Case Else
                    _connected = False
                    Return False
            End Select
        Else
            Err.Raise(vbObjectError + 513, , "Only one Object of the K8055 class can be connected to the card " & _
                                             "at a time! Second connection refused!")
            Return False
        End If
    End Function

    ' Disconnect from the card
    Public Sub disconnect()
        If _connected Then instanceCount -= 1
        CloseDevice()
        _connected = False
    End Sub

    ' Get the version of the DLL
    Public Function getVersion() As Integer
        Return Version
    End Function
#End Region




#Region "Analog Output Channels"
    ' Set all Analog Output Channels to 255
    Public Sub setAllAnalogOutputsToMax()
        SetAllAnalog()
        For i As Integer = 1 To MAX_ANALOG_OUTPUTS
            analogOutputChannels(i) = 255
        Next
    End Sub

    ' Clear all Analog Output Channels
    Public Sub clearAllAnalogOutputs()
        ClearAllAnalog()
        For i As Integer = 1 To MAX_ANALOG_OUTPUTS
            analogOutputChannels(i) = 0
        Next
    End Sub

    ' Set a certain analog channel to 255
    Public Sub setAnalogOutputToMax(ByVal channel As Integer)
        SetAnalogChannel(channel)
        analogOutputChannels(channel) = 255
    End Sub

    ' Set a certain analog channel to 0
    Public Sub cleartAnalogOutput(ByVal channel As Integer)
        ClearAnalogChannel(channel)
        analogOutputChannels(channel) = 0
    End Sub

    ' Set an analog channel to a certain value
    Public Sub setAnalogOutputToValue(ByVal channel As Integer, ByVal value As Integer)
        OutputAnalogChannel(channel, value)
        analogOutputChannels(channel) = value
    End Sub

    ' Set all analog channels to a certain value
    Public Sub setAllAnalogOutputsToValue(ByVal value1 As Integer, ByVal value2 As Integer)
        OutputAllAnalog(value1, value2)
        analogOutputChannels(1) = value1
        analogOutputChannels(2) = value2
    End Sub

    ' Get the value of a certain analog channel
    Public Function getAnalogOutput(ByVal channel As Integer) As Integer
        Return analogOutputChannels(channel)
    End Function
#End Region




#Region "Analog Input Channels"
    ' Reads the selected analog input
    Public Function ReadAnalogInput(ByVal channel As Integer)
        If _timerEnabled And _cacheEnabled Then
            ' Cache mode enabled
            Return analogInputChannels(channel)
        Else
            ' Cache mode disabled
            Return ReadAnalogChannel(channel)
        End If
    End Function

    ' Reads all analog inputs 
    Public Sub readAllAnalogInputs(ByRef data1 As Integer, ByRef data2 As Integer)
        If _timerEnabled And _cacheEnabled And Not _initState Then
            ' Cache mode enabled
            data1 = analogInputChannels(1)
            data2 = analogInputChannels(2)
        Else
            ' Cache mode disabled
            ReadAllAnalog(data1, data2)
        End If

    End Sub
#End Region




#Region "Class Initialisation & Event Generation"
    ' Shared constructor
    ' Will be called only once during the lifetime of the program
    Shared Sub New()
        instanceCount = 0
    End Sub

    ' Default Constructor
    Public Sub New()
        _timerEnabled = False
        initializeTimer()
    End Sub

    ' Default Constructor + EnableEvents Option
    Public Sub New(ByVal enableEvents As Boolean)
        _timerEnabled = enableEvents
        initializeTimer()
    End Sub

    ' Default Constructor + EnableEvents Option + Interval Option 
    Public Sub New(ByVal enableEvents As Boolean, ByVal interval As Integer)
        _timerEnabled = enableEvents
        _timerInterval = interval
        initializeTimer()
    End Sub

    ' Initialize the timer and run it!
    Private Sub initializeTimer()
        readTimer = New Timer()
        readTimer.Enabled = True
        readTimer.Interval = _timerInterval
    End Sub



    '
    ' This function will read the K8055 card at a regular interval. 
    ' It will cache the read values and will launch the events - if applicable.
    ''
    Private Sub readTimer_Cacher(ByVal sender As Object, ByVal e As System.EventArgs) Handles readTimer.Tick
        ' Only check the card if a K8055 card was connected.
        If _timerEnabled And _connected Then
            ' Erase the initState indication.
            _initState = False

            ' Part 1: read all of the digital informations 
            Dim digitalData As Integer
            digitalData = ReadAllDigital()
            Dim digitalChangeBuffer(MAX_DIGITAL_INPUTS) As Integer
            For i As Integer = 1 To MAX_DIGITAL_INPUTS
                Dim currentBit As Integer
                currentBit = (digitalData And Math.Pow(2, i - 1)) > 0
                digitalChangeBuffer(i) = digitalInputChannels(i) <> currentBit
                digitalInputChannels(i) = currentBit
            Next

            ' Part 2: read all of the analog informations
            Dim analogData As Integer
            Dim analogChangeBuffer(MAX_ANALOG_INPUTS) As Integer
            For i As Integer = 1 To MAX_ANALOG_INPUTS
                analogData = ReadAnalogChannel(i)
                analogChangeBuffer(i) = analogData <> analogInputChannels(i)
                analogInputChannels(i) = analogData
            Next

            ' Check if any Digital Input channels were changed
            For i As Integer = 1 To MAX_DIGITAL_INPUTS
                If digitalChangeBuffer(i) Then
                    RaiseEvent digitalInputChanged(i, digitalInputChannels(i))
                End If
            Next

            ' Check if any Analog Input channels were changed
            For i As Integer = 1 To MAX_ANALOG_INPUTS
                If analogChangeBuffer(i) Then
                    RaiseEvent AnalogInputChanged(i, analogInputChannels(i))
                End If
            Next

            ' Fire an additional event to indicate the processing is completed
            RaiseEvent readCycleCompleted()
        End If
    End Sub

#End Region
End Class
